/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with main.c; if not, write to:
 *            The Free Software Foundation, Inc.,
 *            59 Temple Place - Suite 330,
 *            Boston,  MA  02111-1307, USA.
 */


//
// Class: suffix_node
//
// Created by: bountykiller <masse_nicolas@yahoo.fr>
// Created on: Sat Jun 11 00:28:59 2005
//

#include <string.h>
#include "suffix_node.h"
#include "debug.h"


namespace cpl
{
namespace internals
{

template <class T> 
SuffixNode<T>::SuffixNode(const char *key)
{
	TRACE(key);
	if(key)
	{
		localKey = (char*) malloc( (strlen(key)+1) * sizeof(char));
		strcpy(localKey, key);
	}
	isKey = false;
	parent = NULL;
}

template <class T> 
SuffixNode<T>::SuffixNode(const char *key, SuffixTree<T> &root)
{
	TRACE(key);
	localKey = (char*) malloc( (strlen(key)+1) * sizeof(char));
	localKey = strcpy(localKey, key);
	isKey = false;
	parent = &root;
}

template <class T> 
SuffixNode<T>::~SuffixNode()  
{
	ASSERT(localKey); 
	free(localKey);
}

template <class T> 
char SuffixNode<T>::first()
{
	if (localKey) 
		return localKey[0];
	else return '\n';
}

template <class T> 
bool SuffixNode<T>::key_exist(const char *key)
{
	TRACE3(key, localKey, isKey);
	int i=0;
	while(key[i] == localKey[i]) 
	{
		if (!key[i]) 
			return isKey;
		i++;
	}
	if (!localKey[i])
	{
		SuffixNode<T> *node = this->get_subnode(key[i]);
		if (node)
			return node->key_exist(key+i);
	}
	return false;
}


template <class T> 
SuffixNode<T>& SuffixNode<T>::insert(T& data, const char key[])
{
	TRACE2(key, localKey);
	int i = 0;
	while(key[i] == localKey[i]) 
	{
		if (!key[i])
		{
			datas.insert(datas.begin(), data);
			isKey = true;
			return *this;
		}
		i++;
	}
	
	if (!localKey[i]) 
	{
		SuffixNode<T>* node = this->get_subnode(key[i]);
		if (!node) 
		{
			node = new SuffixNode<T>(key+i, *this);
			this->subNodes.insert(this->subNodes.begin(), node);
		}
		return node->insert(data, key+i);			
	}
	else 
	{
		return split(i).insert(data, key);
	}
}

template <class T> 
void SuffixNode<T>::print_tree()
{
//		TRACE(localKey);
	cout << "<" << localKey << ">";
	SuffixTree<T>::print_tree();
	cout << "</" << localKey << ">\n";
}

template<class T> 
SuffixNode<T>& SuffixNode<T>::split(int i)
{
	TRACE2(i, localKey);
	SuffixNode<T>* node;
	char *tmp = (char*) malloc((i+1)*sizeof(char));
	
	strncpy(tmp, localKey, i);
	node = new SuffixNode<T>(tmp, *parent); 
	parent->remove_child(*this);
	parent->add_child(*node);
	node->add_child(*this);
	parent = node;
	free(tmp);
	tmp = (char*) malloc(strlen(localKey+i)+1 * sizeof(char));
	strcpy(tmp, localKey+i);
	free(localKey);
	localKey = tmp;

	return *node;
}


}
}//namespaces




