/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with main.c; if not, write to:
 *            The Free Software Foundation, Inc.,
 *            59 Temple Place - Suite 330,
 *            Boston,  MA  02111-1307, USA.
 */


//
// Class: suffix_tree
//
// Created by: bountykiller <masse_nicolas@yahoo.fr>
// Created on: Sat Jun 11 00:16:16 2005
//

#include "suffix_tree.h"
#include "suffix_node.h"
#include "debug.h"

namespace cpl
{
namespace internals
{

template<typename T> 
SuffixTree<T>::~SuffixTree()
{
	for(typename SuffixTree<T>::SuffixNodePlist::iterator iter = subNodes.begin();
		iter != subNodes.end; iter = subNodes.erase(iter))
		delete &(*iter);
}


template<typename T> 
SuffixNode<T>& SuffixTree<T>::insert(T& data, const char *key)
{
	TRACE(key);
	SuffixNode<T> *node = this->get_subnode(key[0]);
	if(!node)
	{
		node = new SuffixNode<T>(key, *this);
		subNodes.insert(subNodes.begin(), node);
	}
	return node->insert(data, key);
}


template<typename T> 
SuffixNode<T>& SuffixTree<T>::remove(T& data, const char *key)
{
	SuffixNode<T> *node = get_subnode(key[0]);
	if(!node) return;
	return node->remove(data, key);
}


template<typename T> 
list<T&> SuffixTree<T>::remove(const char *key)
{
	SuffixNode<T> *node = get_subnode(key[0]);
	if(!node) return ;
	return node->remove(key);
}


template<typename T> 
bool SuffixTree<T>::key_exist(const char *key)
{
	TRACE(key);
	SuffixNode<T> *node = get_subnode(key[0]);
	
	if (node)
		return node->key_exist(key);
	return false;
}


template<typename T> 
void SuffixTree<T>::add_child(SuffixNode<T> &node)
{
	TRACE(node.first());
	subNodes.insert(subNodes.begin(), &node);
}


template<typename T> 
void SuffixTree<T>::remove_child(SuffixNode<T> &node)
{
	TRACE(node.first());
	typename SuffixTree<T>::SuffixNodePlist::iterator iter = subNodes.begin();
	while(iter != subNodes.end())
	{
		if ((*iter)->first() == node.first())
			iter = subNodes.erase(iter);
		else iter++;	
	}
}


template<typename T> 
void SuffixTree<T>::print_tree()
{
	TRACE("");
	
	for(typename SuffixTree<T>::SuffixNodePlist::iterator iter = subNodes.begin();
		iter != subNodes.end(); ++iter)
	{
		(*iter)->print_tree();
	}
}

template<typename T> 
SuffixNode<T>* SuffixTree<T>::get_subnode(char key)
{
	TRACE(key);
	typename SuffixTree<T>::SuffixNodePlist::iterator iter = subNodes.begin();
    while (iter != subNodes.end())
    {
        if ((*iter)->first() == key)
			return (*iter);
        ++iter;
    }
	return NULL;
}


/*
template <class T> 
SuffixNode& SuffixTree<T>::split(int i)
    {
	TRACE2(i, localKey);
	SuffixNode* node;
	char *tmp = (char*) malloc((i+1)*sizeof(char));
	tmp = strncpy(tmp, localKey, i);
	node = new SuffixNode(localKey+i, *this); 
	subNodes.insert(subNodes.begin(), node);
	free(localKey);
	localKey = tmp;
	return *node;
}
*/	

}
}//namespaces
